package hr.pmf.abkloubcar.drd;

import hr.pmf.abkloubcar.drd.model.Node;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class DominatingSetSolver {

    public enum DominatingSetSort {
        DEFAULT("DEFAULT"),
        DEGREE_DESC("DEGREE_DESC"),
        COST_ASC("COST_ASC"),
        COST_DEGREE_FACTOR_DESC("COST_DEGREE_FACTOR_DESC");

        private final String text;

        DominatingSetSort(final String text) {
            this.text = text;
        }

        @Override
        public String toString() {
            return text;
        }

    }

    public enum DominatingSetSelection {
        NAIVE("NAIVE"),
        DEGREE("DEGREE");

        private final String text;

        DominatingSetSelection(final String text) {
            this.text = text;
        }

        @Override
        public String toString() {
            return text;
        }
    }

    public static void process(List<Node> graph, DominatingSetSelection variant, DominatingSetSort sort) {
        switch (sort) {
            case DEGREE_DESC -> {
                graph.sort(Comparator.comparing(Node::getDegree));
                Collections.reverse(graph);
            }
            case COST_ASC -> {
                graph.sort(Comparator.comparing(Node::getCost));
            }
            case COST_DEGREE_FACTOR_DESC -> {
                graph.sort(Comparator.comparing(Node::getDegreeCostFactor));
                Collections.reverse(graph);
            }
        }

        switch (variant) {
            case NAIVE -> {
                naiveSet(graph);
            }
            case DEGREE -> {
                degreeSet(graph, 2);
            }
        }
    }

    private static void setAsDominating(Node node) {
        node.dominating = true;
        node.dominated = false;
        node.neighbours.forEach(neighbour -> neighbour.dominated = true);
    }

    private static void naiveSet(List<Node> graph) {
        for (var node : graph) {
            if (!node.dominated) {
                setAsDominating(node);
            }
        }
    }

    private static void degreeSet(List<Node> graph, int factor) {
        for (var node : graph) {
            var degreeNotDominated = node.neighbours.stream().filter(x -> !x.dominated && !x.dominating).count();
            if (!node.dominated || degreeNotDominated >= factor) {
                setAsDominating(node);
            }
        }
    }
}
