package hr.pmf.abkloubcar.drd;

import hr.pmf.abkloubcar.drd.model.Node;

import java.util.List;

public class ImprovementSolver {

    private static boolean isDominated(List<Node> neighbours, int weight) {

        var threes = neighbours.stream().filter(x -> x.weight == 3).count();
        var twos = neighbours.stream().filter(x -> x.weight == 2).count();
        if (weight == 3) {
            return true;
        }
        if (weight == 2) {
            return true;
        }
        if (weight == 1) {
            return (threes + twos) > 0;
        }
        if (weight == 0) {
            return threes > 0 || twos > 1;
        }
        return false;
    }

    private static boolean areAllDominated(List<Node> nodes) {
        var possible = true;
        for (Node node : nodes) {
            if (!isDominated(node.neighbours, node.weight)) {
                possible = false;
                break;
            }
        }
        return possible;
    }

    private static boolean reduceWeight(Node node) {
        var success = false;
        if (node.weight == 0) {
            return false;
        }
        if (node.weight == 1) {
            if (isDominated(node.neighbours, 0)) {
                node.weight = 0;
                success = true;
            }
            return success;
        }
        for (var weight = 0; weight < node.weight; weight++) {
            if (isDominated(node.neighbours, weight)) {
                var oldWeight = node.weight;
                node.weight = weight;
                if (areAllDominated(node.neighbours)) {
                    success = true;
                    break;
                } else {
                    node.weight = oldWeight;
                }
            }
        }
        return success;
    }

    private static boolean transferWeightEffective(Node node, Node neighbour, int weightChange, int bestSavings) {
        if (neighbour.weight + weightChange > 3) {
            return false;
        }
        if (neighbour.cost > node.cost) {
            return false;
        }
        var success = false;
        var oldWeight = node.weight;
        var oldNeighbourWeight = neighbour.weight;
        neighbour.weight += weightChange;
        node.weight -= weightChange;
        if (areAllDominated(neighbour.neighbours) && isDominated(neighbour.neighbours, neighbour.weight) && areAllDominated(node.neighbours)) {
            var neighbourCost = neighbour.weight * neighbour.cost;
            var savings = node.cost * weightChange - neighbourCost * weightChange;
            if (savings > bestSavings) {
                success = true;
            }
        }
        node.weight = oldWeight;
        neighbour.weight = oldNeighbourWeight;
        return success;
    }

    private static boolean transferWeight(Node node) {
        var success = false;
        if (node.weight == 0) {
            return false;
        }
        if (node.weight == 1) {
            return false;
        }
        Node bestNode = null;
        int bestWeight = 0;
        int bestSavings = 0;
        for (Node neighbour : node.neighbours) {
            for (var weightChange = 1; weightChange <= node.weight; weightChange++) {
                if (transferWeightEffective(node, neighbour, weightChange, bestSavings)) {

                    bestNode = neighbour;
                    bestWeight = weightChange;
                    bestSavings = node.cost * weightChange - neighbour.cost * weightChange;
                }
            }
        }
        if (bestNode != null) {

            bestNode.weight += bestWeight;
            node.weight -= bestWeight;
            success = true;
        }

        return success;
    }

    public static void improve(List<Node> graph) {

        var success = true;
        while (success) {
            success = false;
            for (Node node : graph) {
                if (node.weight == 0) {
                    continue;
                }
                if (reduceWeight(node)) {
                    success = true;
                }
                if (transferWeight(node)) {
                    success = true;
                }
            }
        }
    }
}
