package hr.pmf.abkloubcar.drd.generator;

import hr.pmf.abkloubcar.drd.util.Util;

import java.io.IOException;
import java.util.*;

import static hr.pmf.abkloubcar.drd.util.Util.*;

public class GraphGenerator {

    public static void generateGraph(int size, double density, int numberOfGraphs) throws IOException {


        for (var graphIndex = 0; graphIndex < numberOfGraphs; graphIndex++ ) {
            var maxNumberOfEdges = (size * (size - 1)) / 2;
            var numberOfEdges = Math.round(maxNumberOfEdges * density);
            log("Generating graph " + size + " " + numberOfEdges + " " + density + " " + maxNumberOfEdges);
            var file = "graph_" + size + "_" + Math.round((density * 100)) + "_" + graphIndex + ".txt";
            Util.createFileIfNotExisting(INPUT_FOLDER, file);
            resetFile(INPUT_FOLDER, file);

            appendToFile(INPUT_FOLDER, file, size + " " + numberOfEdges, true);

            Random rand = new Random();

            var generatedEdges = new ArrayList<String>();
            while (generatedEdges.size() < numberOfEdges) {
                var nodeA = rand.nextInt(size);
                var nodeB = rand.nextInt(size);
                var generatedEdge = nodeA > nodeB ? nodeB + " " + nodeA : nodeA + " " + nodeB;

                if (nodeA != nodeB && !generatedEdges.contains(generatedEdge)) {
                    generatedEdges.add(generatedEdge);
                    appendToFile(INPUT_FOLDER, file, "e " + generatedEdge, true);
                }
            }

            for (var i = 0; i < size; i++) {
                var weight = rand.nextInt(size + 1);
                var line = i == 0 ?  weight  + "" : " " + weight;
                appendToFile(INPUT_FOLDER, file, line, false);
            }
            log("Graph generated");
        }
    }

    public static void main(String[] args) throws IOException {
        generateGraph(500, 0.20, 30);
        //generateGraph(50, 0.50, 30);
    }
}
